#ifndef SAW_PLOTTER_PLOTTER
#define SAW_PLOTTER_PLOTTER

// root includes
#include "w32pragma.h"
#include "TApplication.h"
#include "TCanvas.h"
#include "TGraph.h"
#include "TH1D.h"
#include "TF1.h"
#include "TThread.h"
#include "TMarker.h"
#include "TRandom3.h"

// other includes (have to come after root includes)
#include <vector>
#include <boost/smart_ptr.hpp>

namespace AWPlotter
{
    class Plotter
    {
    public:

        //Types
        typedef boost::shared_ptr<TCanvas>  TCanvasPtr;
        typedef boost::shared_ptr<TH1D>     TH1DPtr;
        typedef boost::shared_ptr<TGraph>   TGraphPtr;

        // Construction
        Plotter();
        virtual ~Plotter();

        // Accessors
        bool start() {return m_start;}

        // Operations
        void init(int numBins);
        void setStart(bool start) {m_start = start;}
        void plotHistogram(double data);
        void plotScatter(const std::vector<double>& X, const std::vector<double>& Y);
        TVirtualPad* pad(int i);

    private:        

        // Data
        double                  m_min;      //min value seen so far
        double                  m_max;      //max value seen so far
        bool                    m_start;

        TCanvas*                m_c1;
        TCanvas*                m_c2;
        TH1DPtr                 m_hist;
        TGraphPtr               m_graph;
        
    };
    typedef boost::shared_ptr<Plotter> PlotterPtr;
}

#endif